/*******************************************************************************
  Project name: FFT Demo

 Device name: dsPIC33FJ256GP710A
 http://ww1.microchip.com/downloads/en/DeviceDoc/70593d.pdf
 Device clock: 80.000000 MHz
 Dev.Board: EasyPIC Fusion v7
 http://www.mikroe.com/easypic-fusion/

 This project is designed to work with dsPIC33FJ256GP710A. It has been tested
   on EasyPIC Fusion v7 board with 10MHz crystal. It should work with any
   other crystal, just make sure that the operating frequency is greater than
   60MHz. For example, if you are using 10MHz crystal, make sure you turn 8xPLL
   on, in that way the operating frequency will be 80MHz. Note: the maximum
   operating frequency for dsPIC is 120MHz.
 With minor adjustments, this example should work with any other dsPIC MCU
   that has one AD converter.

 This code demonstrates how to calculate/interpret FFT (DFT) of an analogue signal.

 Basic concept:
   Input signal is sampled at regular intervals and stored into
     a buffer. The buffer is interpreted as a complex array of samples,
     where the Real part is an actual sample, while the Imaginary part is equal
     to zero.
   Once the buffer is filled with samples, it is passed to the FFT procedure.
   The FFT procedure calculates the Fast (Discrete) Fourier Transformation of the input
     signal and returns back the FFT samples into the input buffer.
   The FFT samples are also given in format Re,Im,Re,Im..., but the Im part is
     now different from zero.
   The amplitude of each FFT sample is calculated as F[k] = sqrt(Re[k]^2+Im[k]^2).
   Since the FFT spectrum is symmetrical, only the first half of the input buffer
     is considered.
   The amplitude spectrum is drawn on GLCD. The whole process repeats infinitely,
     giving impression that the spectrum is drawn in realtime.
   Signal power can be calculated as a sum of all FFT samples:
     Pw = sum(k=0..N-1)(F[k])

 Input signal requirements: The input signal is an analog signal, the
   maximum frequency of which does not exceed 35kHz. Its amplitude is 1V at most,
   while its DC value is 1V as well. Values other than this may cause saturation
   and output signal distortion. If you experience strange drawings on GLCD, try
   with altering input signal levels until the image stabilizes.
   In order to analyze signals faster than 35kHz, input array buffer must be increased.
   All the same, the sampling frequency should be at least 2*Fmax (Niquist criteria).

  Version History:
   * 2005-12-02 (ZR)
     - Initial version
   * 2009-11-24 (ZR)
     - Modification for mikroC PRO for dsPIC compiler
     - Introduced Timer1 interrupt for sampling the input signal
     - Increased sampling frequency
     - Spectrum drawn over entire GLCD area

*******************************************************************************/

const unsigned long NUM_OF_SAMPLES = 256;// 256 pairs of [Re, Im] values
const unsigned long SAMPLE_FREQ  = 70000;// kHz

  unsigned Samples[NUM_OF_SAMPLES * 2] ;
  ydata unsigned InputSamples[NUM_OF_SAMPLES * 2];  // Y data space is required by FFT routine
                                           //   See datasheet for your dsPIC to see Y data space limits.
  unsigned freq;                           // Auxiliary variables
  char txt[10];
  unsigned Written[128];
  unsigned globali;
  char doDraw;

// Glcd module connections
sbit GLCD_D0 at RE0_bit;
sbit GLCD_D1 at RE1_bit;
sbit GLCD_D2 at RE2_bit;
sbit GLCD_D3 at RE3_bit;
sbit GLCD_D4 at RE4_bit;
sbit GLCD_D5 at RE5_bit;
sbit GLCD_D6 at RE6_bit;
sbit GLCD_D7 at RE7_bit;

sbit GLCD_D0_Direction at TRISE0_bit;
sbit GLCD_D1_Direction at TRISE1_bit;
sbit GLCD_D2_Direction at TRISE2_bit;
sbit GLCD_D3_Direction at TRISE3_bit;
sbit GLCD_D4_Direction at TRISE4_bit;
sbit GLCD_D5_Direction at TRISE5_bit;
sbit GLCD_D6_Direction at TRISE6_bit;
sbit GLCD_D7_Direction at TRISE7_bit;

sbit GLCD_CS1 at LATD5_bit;
sbit GLCD_CS2 at LATD4_bit;
sbit GLCD_RS  at LATD9_bit;
sbit GLCD_RW  at LATG6_bit;
sbit GLCD_EN  at LATD10_bit;
sbit GLCD_RST at LATD7_bit;

sbit GLCD_CS1_Direction at TRISD5_bit;
sbit GLCD_CS2_Direction at TRISD4_bit;
sbit GLCD_RS_Direction  at TRISD9_bit;
sbit GLCD_RW_Direction  at TRISG6_bit;
sbit GLCD_EN_Direction  at TRISD10_bit;
sbit GLCD_RST_Direction at TRISD7_bit;
// End Glcd module connections

//-------------- Initialization of AD converter
void InitAdc() {

  //ADPCFG = 0x00FF; // PORTB<8:15> is analog, PORTB<0:7> is digital
//  TRISB.F8 = 1;    // RB8 as input pin
  AD1PCFGL = 0xFFFF; // Set All pins as digital
  AD1PCFGH = 0xFFFF;

  TRISB.F8 = 1;


  AD1CHS0  = 8;      // Connect RBxx/ANxx as CH8 input. RB8 is input pin
  AD1CSSL = 0;      //
  AD1CON3 = 0x001F; // sample time = 31 Tad.
  AD1CON2 = 0;
  AD1CON1 = 0x8304; // turn ADC ON, fractional result, manual conversion start, auto sampling

}//~

//-------------- Initialize GLCD for dsPICPRO board
void InitGlcd() {
  Glcd_Init();
  delay_ms(100);
  Glcd_Fill(0xAA); // Show stripes on GLCD to signalize startup
  Delay_ms(500);   // Wait for a while
  Glcd_Fill(0x00); // Clear screen
}//~

//-------------- Main Initialization
void MainInit() {
    // PLL settings
  CLKDIVbits.PLLPRE = 0;      // PLLPRE<4:0> = 0  ->  N1 = 2    8MHz / 2 = 4MHz
                              // (must be within 0.8 MHz to 8 MHz range)
  PLLFBD =   38;              // PLLDIV<8:0> = 38 ->  M = 40    4MHz * 40 = 160MHz
                              // (must be within 100 MHz to 200 MHz range)
  CLKDIVbits.PLLPOST = 0;     // PLLPOST<1:0> = 0 ->  N2 = 2    160MHz / 2 = 80MHz
                              // (must be within 12.5 MHz to 80 MHz range)

  globali = 0;
  DoDraw = 0;
  TRISF = 0;
  LATF  = 0;
  memset(InputSamples, 0, 1024); // Clear samples
  InitAdc();
  InitGlcd();
  Vector_Set(Written, 128, 0xFFFF);  // Fill "Written" with $FFFF
  Glcd_Write_Text(" Hz", 100, 0, 1);

  // prescaler 1:1
  T1CON.B5 = 0;
  T1CON.B4 = 0;
  PR1 = (unsigned long)(Get_Fosc_kHz()) * 1000 / (2 * SAMPLE_FREQ);
  TON_bit = 1; // start timer1
  IEC0.T1IE = 1; // enable T1 interrupt
  
  

  T1CON	 = 0x8000;
  T1IE_bit	 = 1;
  T1IF_bit	 = 0;
  IPC0		 = IPC0 | 0x1000;
  PR1		 = 571;
  
}//~


//-------------- Auxiliary function for converting 1.15 radix point to
//               IEEE floating point variable (needed for sqrt).
float Fract2Float(int input) {

   if (input < 0)
     input = - input;
   return (input / 32768.);
}//~


//-------------- Data output procedure. It draws FFT components on GLCD.
//               GLCD coordinate system starts at top left corner. Therefore,
//               line drawing had to be modified in order to achieve
//               a viewable spectrum on screen.
//               "Samples" at this moment contains DFT of the signal in the manner Re, Im, Re, Im...
void WriteData() {

  unsigned Re, Im, tmpw,
           j, k, l, max;
  float    Rer, Imr, tmpR;


  j      = 0;                      // If you want to skip DC component then make j >= 1
  k      = 0;
  max    = 0;
  freq = 0;                        // Reset current max. frequency for new reading




  while (k <= 127)   {
      Re = InputSamples[j++];      // Real part of DFT sample

      Im = InputSamples[j++];      // Imaginary part of DFT sample

      Rer = Fract2Float(Re);       // conversion to IEEE floating point
      Imr = Fract2Float(Im);       // conversion to IEEE floating point

      tmpR = Rer * Rer;            // Re^2
      Rer  = tmpR;
      tmpR = Imr * Imr;            // Im^2
      Imr  = tmpR;
      tmpR = sqrt(Rer + Imr);      // Amplitude of current DFT sample
      Rer  = tmpR * 512.;          // DFT is scaled down by 1/N, we need to
                                   //   take it back in order to have visible
                                   //   components on GLCD
      Re = Rer;                    // Truncate to nearest integer

      if (Re > 63)
        if(k != 0)
          //--- NOTE: rejecting values for k=0 removes strong DC component
          Re = Written[k-1];       // k = 0? beware of the glitch
        else
          Re = 0;

      if (Re > max) {
          max = Re;
          freq = k;                // This should be the center frequency of the signal
      }

      tmpw = Written[k];
      if (tmpw != Re) {            // Draw only those components that changed
          l = 64 - tmpw;           // 64 lines on GLCD on Y axis
          while (l <= 63) {        // Clear line to the bottom of the screen
             Glcd_Dot(k, l, 0);
             l++;
          }

           l = 64 - Re;           // Draw line to the bottom of the screen
           while (l <= 63) {
               Glcd_Dot(k, l, 1);
               l++;
           }
           Written[k] = Re;       // Mark that the current sample has been drawn
      }
      k++;                      // Move current X coordinate
   }

   //--- Write the frequency of max. amplitude sample
   freq *= (SAMPLE_FREQ / NUM_OF_SAMPLES);
   WordToStr(freq, txt);
   Glcd_Write_Text(txt, 70, 0, 1);
}//~

//-------------- Takes current sample
unsigned ReadAdc() {
  LATF = ~ PORTF;               // you can put osciloscope on PORTF to check the sampling frequency
  SAMP_bit = 0;                    // Start conversion
  while (DONE_bit == 0)            // Wait for ADC to finish
    asm nop;
  return ADCBUF0;                   // Get ADC value
}//~


//-------------- Fills "Samples" with input samples in manner Re, Im, Re, Im... where Im = 0
void SampleInput()
 {
   Samples[globali++] = ReadAdc();   // Re
   Samples[globali++] = 0;            // Im
   if (globali >= 511)
     {   globali = 0;
         if (!doDraw)
         { //do not overwrite non processed
           memcpy(InputSamples, Samples, 1024);
           DoDraw = 1;
         }
     }
  // "Samples" now contains 128 pairs of <Re, Im> samples
}//~


void Timer1Int() iv IVT_ADDR_T1INTERRUPT{ // sample with 20kHz
  SampleInput();              // Sample input signal
  LATF = ~PORTF;
  IFS0.T1IF = 0; // clear interrupt flag
 }

//-------------- Main program starts here
void main()
 {
  MainInit();                     // Initialize all
  while (1)                      // Infinite loop
  {
     if (doDraw) {
      // Perform FFT (DFT), 7 stages, 128 samples of complex pairs
      // Twiddle factors are taken from the <TwiddleFactors.dpas> unit
      FFT(8, TwiddleCoeff_256, InputSamples);
      // DFT butterfly algorythm bit-reverses output samples.
      // We have to restore them in natural order.
      BitReverseComplex(8, InputSamples);
      // Draw DFT samples on GLCD
      WriteData();
      doDraw = 0;
     }
  }
 }//~!