/*
 This code demonstrates how to filter an analogue signal using FIR filter.
 Filter coefficients are calculated by the Filter Designer Tool,
   available in mikroC menu (Tools-> Filter Designer). This tool also
   calculates the value of PR1 register for wanted sampling rate.

 Basic concept: Input signal is sampled at regular intervals and stored into
   a circular buffer. The buffer is then passed through FIR filter which in
   return calculates the value of a current sample. That value is then sent to
   DAC. DAC output may look not as smooth as input signal, which depends on
   DAC resolution. In this example Microchip MCP 4921 DAC is used
   (10 bits resolution). In case the output signal is too stairy, an analogue
   filter should be put on DAC output which will cut upper frequencies of the
   input signal.

 Input signal requirements: The input signal is an analogue signal, the
   maximum frequency of which does not exceed 15kHz. Its amplitude is 500mV at most,
   while its DC value is 500mV. Values other than this may cause saturation
   and output signal distortion.
     Sampling Frequency: 20000 Hz
 Filter setup:
     Filter kind: FIR
     Filter type: Lowpass filter
     Filter order: 20
     Filter window: Bartlett (Triangle)
     Filter borders:
       Wpass:4000 Hz

 Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593d.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     Ext. Modules:    mE ac:DAC_SPI
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
*/

const unsigned BUFFFER_SIZE  = 32;
const unsigned FILTER_ORDER  = 20;

const unsigned CS_PIN        = 1;  // DAC CS pin

const  COEFF_B[FILTER_ORDER+1] = {
      0x0000, 0x0048, 0x0133, 0x02D3, 0x052B, 0x0826,
      0x0BA0, 0x0F62, 0x1329, 0x16AA, 0x199A, 0x16AA,
      0x1329, 0x0F62, 0x0BA0, 0x0826, 0x052B, 0x02D3,
      0x0133, 0x0048, 0x0000
  };
      
unsigned inext;                       // Input buffer index
ydata unsigned input[BUFFFER_SIZE];   // Input buffer, must be in Y data space

/* This is ADC interrupt handler.
 Analog input is sampled and the value is stored into input buffer.
 Input buffer is then passed through filter.
 Finally, the resulting output sample is sent to DAC.
 */
void ADC1Int() iv IVT_ADDR_ADC1INTERRUPT {                // ADC interrupt handler
  unsigned CurrentValue;
  input[inext] = ADCBUF0;                 // Fetch sample

  CurrentValue = FIR_Radix(FILTER_ORDER+1,// Filter order
                            COEFF_B,      // b coefficients of the filter
                            BUFFFER_SIZE, // Input buffer length
                            input,        // Input buffer
                            inext);       // Current sample

  inext = (inext+1) & (BUFFFER_SIZE-1);   // inext = (inext + 1) mod BUFFFER_SIZE;

  LATG.CS_PIN = 0;                        // CS enable for DAC
  SPI2_Write(0x3000 | CurrentValue);      // Write CurrentValue to DAC (0x3 is required by DAC)
  LATG.CS_PIN    = 1;                     // CS disable for DAC

  IFS0.F13 = 0;                           // Clear AD1IF
}//~


/* This is Timer1 interrupt handler.
   It is used to start ADC at periodic intervals.
 */
void Timer1Int() iv IVT_ADDR_T1INTERRUPT {              // Timer1 interrupt handler

 if (AD1CON1.DONE == 1){                 // If ADC is not busy
    AD1CON1.SAMP  = 1;                    // Start new sample
  }
  LATD = PORTD ^ 0xFFFF;

  IFS0.F3    = 0;
}//~

/* The main program starts here.
   Firstly, hardware peripherals are initialized and then
   the program goes to an infinite loop, waiting for interrupts. */
void main() {

  // PLL settings
  CLKDIVbits.PLLPRE = 0; // PLLPRE<4:0> = 0 -> N1 = 2 8MHz / 2 = 4MHz
  // (must be within 0.8 MHz to 8 MHz range)
  PLLFBD = 38; // PLLDIV<8:0> = 38 -> M = 40 4MHz * 40 = 160MHz
  // (must be within 100 MHz to 200 MHz range)
  CLKDIVbits.PLLPOST = 0; // PLLPOST<1:0> = 0 -> N2 = 2 160MHz / 2 = 80MHz
  // (must be within 12.5 MHz to 80 MHz range)

  TRISD = 0;
  // DAC setup
  TRISG.CS_PIN   = 0;                    // CS pin
  LATG.CS_PIN    = 1;                    // Set CS to inactive

  // SPI setup
  SPI2_Init_Advanced(_SPI_MASTER, _SPI_16_BIT, _SPI_PRESCALE_SEC_2, _SPI_PRESCALE_PRI_16, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE);
                     
  inext  = 0;                            // Initialize buffer index
  Vector_Set(input, BUFFFER_SIZE, 0);    // Clear input buffer
  
  TRISB   = 0xFFFF;                      // Use PORTB for input signal
  AD1CON1  = 0x00E2;                      // Auto-stop sampling, unsigned integer out
  AD1CON2  = 0x0000;
  AD1CON3  = 0x021A;                      // Sampling time= 3*Tad, minimum Tad selected
  ADPCFG  = 0x0000;                      // Configure PORTB as ADC input port
  AD1CHS0   = 0x000A;                      // Sample input on RB10
  AD1CSSL  = 0;                           // No input scan
  ADPCFG  = 0x0000;                      // Configure PORTB as analog ADC input port

  // Interrupts setup
  IFS0    = 0;
  IFS1    = 0;
  IFS2    = 0;
  INTCON1 = 0x8000;                       // Nested interrupts DISABLED
  INTCON2 = 0;
  IEC0    = 0x2008;                       // Timer1 and ADC interrupts ENABLED
  IPC0.F12= 1;                            // Timer1 interrupt priority level = 1
  IPC3.F4 = 1;                            // ADC interrupt priority level = 1

  PR1     = 250;                         // Sampling ~= 80 kHz. The value of 250 is calculated for 80MHz clock.
  T1CON   = 0x8000;                      // Start Timer1, internal clock FCY, prescaler 1:1

  AD1CON1.F15 = 1;
  AD1CON1.SAMP  = 1;
  while (1);                             // Infinite loop,
                                         //   wait for interrupts
}//~!