/*
 * Project name:
     Signal_Generator
 * Copyright:
     (c) MikroElektronika, 2009.
 * Revision History:
     20091030:
       - Initial release
 * Description:
     The DDS method allows generation a very precise signal frequency Fs.
     The general idea is to use an accumulator to store the shift value DELTA.
     The accumulator can be divided in two parts, A and B. The part A represents
     index of a look-up table which contains signal samples. The part B (lower
     bits comparing to A) defines resolution of frequency tunning: N=2^B. In the
     example below N= 2^16= 65536.
     Frequency of the signal is given by Fs = (DELTA / N) * Fb, where Fb is the
     rate at which DELTA is stored in accumulator.
     Frequency resolution is given by dFs = Fb/N, (DELTA = 1). For the given example
     Fb is limited by DAC speed.
 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593d.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL, 80.000MHz
     Ext. Modules:    mE ac:DAC_SPI
     SW:              mikroC PRO for dsPIC
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     -

*/

// These are samples of the sine signal in the range [0..pi].
// sin(x) is a symmetrical signal, therefore the range [0..pi/4] can be used too. However,
// rotating samples and chosing a proper index takes some time, which limits
// the maximum frequency.
// Samples of any other signal can be used too. Attention should be taken to make sure
// that the first and the last sample need to be similar to each other,
// otherwise discontinuities can happen. If the discontinuities are a wanted feature,
// make sure that amplitude between two consecutive samples is not too large, otherwise
// DAC may not be able to follow such fast transitions.

const unsigned int SineSamples[256] =
  {0,        1,        2,        6,        10,        15,        22,        30,        39,        50,        61,        74,        88,        103,        120,        137,        156,
   176,        197,        219,        242,        266,        291,        318,        345,        374,        403,        433,        465,        497,
   531,        565,        600,        636,        673,        710,        749,        788,        828,        869,        910,        952,        995,
  1039,        1083,        1127,        1172,        1218,        1264,        1311,        1358,        1406,        1453,        1502,        1550,        1599,
  1648,        1698,        1747,        1797,        1847,        1897,        1948,        1998,        2048,        2098,        2148,        2199,        2249,
  2299,        2349,        2398,        2448,        2497,        2546,        2594,        2643,        2690,        2738,        2785,        2832,        2878,
  2924,        2969,        3013,        3057,        3101,        3144,        3186,        3227,        3268,        3308,        3347,        3386,        3423,
  3460,        3496,        3531,        3565,        3599,        3631,        3663,        3693,        3722,        3751,        3778,        3805,        3830,
  3854,        3877,        3899,        3920,        3940,        3959,        3976,        3993,        4008,        4022,        4035,        4046,        4057,
  4066,        4074,        4081,        4086,        4090,        4094,        4095,        4095,        4095,        4094,        4090,        4086,        4081,
  4074,        4066,        4057,        4046,        4035,        4022,        4008,        3993,        3976,        3959,        3940,        3920,        3899,
  3877,        3854,        3830,        3805,        3778,        3751,        3722,        3693,        3663,        3631,        3599,        3565,        3531,
  3496,        3460,        3423,        3386,        3347,        3308,        3268,        3227,        3186,        3144,        3101,        3057,        3013,
  2969,        2924,        2878,        2832,        2785,        2738,        2690,        2643,        2594,        2546,        2497,        2448,        2398,
  2349,        2299,        2249,        2199,        2148,        2098,        2048,        1998,        1948,        1897,        1847,        1797,        1747,
  1698,        1648,        1599,        1550,        1502,        1453,        1406,        1358,        1311,        1264,        1218,        1172,        1127,
  1083,        1039,        995,        952,        910,        869,        828,        788,        749,        710,        673,        636,        600,
  565,        531,        497,        465,        433,        403,        374,        345,        318,        291,        266,        242,        219,
  197,        176,        156,        137,        120,        103,        88,        74,        61,        50,        39,        30,        22,        15,        10,        6,
  2,        1} ;

unsigned CurrentIndex, CurrentValue, delta;

// The following trap procedures are not really needed here, they
//   are used here just for the sake of demonstration.
void OscillatorFailTrap(void) iv IVT_ADDR_OSCILLATORFAIL { // if oscillator fails, the code jumps here
  LATD = 0x5555;
  while(1)
    ;
}

void AddressTrap(void) iv IVT_ADDR_ADDRESSERROR {          // if the addressing mode is wrong, the code jumps here
  LATD = 0xAAAA;
  while(1)
    ;
}

void StackErrorTrap() iv IVT_ADDR_STACKERROR {             // stack overflow, underflow...
  LATD = 0x0F0F;
  while(1)
    ;
}

void MathErrorTrap() iv IVT_ADDR_MATHERROR {               // div by zero etc...
  LATD = 0xA0A0;
  while(1)
    ;
}

// The project has been setup up so that a switch on RA15 (INT4)
//   increases frequency on each rising edge
void Interrupt1() iv IVT_ADDR_INT4INTERRUPT { // RA15(INT4) interrupt
  delta += 10;
  LATD++;           // Signalize the interrupt has happend
  IFS3.INT4IF = 0;  // Clear INT4IF
}

// The project has been setup up so that a switch on RA14 (INT3)
//   decreases frequency on each rising edge
void Interrupt2() iv IVT_ADDR_INT3INTERRUPT { // RA14(INT3) interrupt
   delta -= 10;
   LATD--;          // Signalize the interrupt has happend
   IFS3.INT3IF = 0; // Clear INT3IF
}

// Initialize
void Init() {

  // PLL settings
  CLKDIVbits.PLLPRE = 0; // PLLPRE<4:0> = 0 -> N1 = 2 8MHz / 2 = 4MHz
  // (must be within 0.8 MHz to 8 MHz range)
  PLLFBD = 38; // PLLDIV<8:0> = 38 -> M = 40 4MHz * 40 = 160MHz
  // (must be within 100 MHz to 200 MHz range)
  CLKDIVbits.PLLPOST = 0; // PLLPOST<1:0> = 0 -> N2 = 2 160MHz / 2 = 80MHz
  // (must be within 12.5 MHz to 80 MHz range)

  ADPCFG = 0xFFFF;         // Configure AN pins as digital
  TRISA  = 0xFFFF;         // Set PORTA to input

  // DAC setup
  TRISG1_bit = 0;          // CS pin
  LATG1_bit  = 1;          // Set CS to inactive

  // SPI setup
  SPI2_Init_Advanced(_SPI_MASTER, _SPI_16_BIT, _SPI_PRESCALE_SEC_2, _SPI_PRESCALE_PRI_16, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE);

  IEC3.INT4IE     = 1;     // Enable INT4 interrupt
  IEC3.INT3IE     = 1;     // Enable INT3 interrupt
  INTCON2.INT4EP  = 0;     // Rising edge for INT4
  INTCON2.INT3EP  = 0;     // Rising edge for INT3
  IPC13.INT4IP_0    = 1;   // Interrupt priority for INT4 is 1
  IPC13.INT4IP_1    = 0;   // Interrupt priority for INT4 is 1
  IPC13.INT3IP_0    = 1;   // Interrupt priority for INT3 is 1
  IPC13.INT3IP_1    = 0;   // Interrupt priority for INT3 is 1
  INTCON1.NSTDIS  = 1;     // Interrupt nesting is not allowed
  
  // Clearing accumulator A
  ACCAU = 0;
  ACCAH = 0;
  ACCAL = 0;
  
  // Alternative way of clearing accumulator A:
  /*asm {
    mov #0, w0
    lac w0, A
  } */

  delta = 1024;     // Define signal frequency by setting DELTA
  LATD  = 0;        // Initialize monitor
  TRISD = 0x00;
  CurrentIndex = 0;
}

unsigned GetIndex(unsigned delta) {
  W0 = delta;
  asm {
    add w0, A             // A = A + delta
  }
  return ACCAH >> 8;      // Look-up table is indexed by ACC<31:24> bits
}


volatile unsigned int bla;

void main() {
  Init();
  while(1) {
    CurrentIndex = GetIndex(delta);            // Compute new index for look-up table
    CurrentValue = SineSamples[CurrentIndex];  // Read sample from look-up table

    // Output CurrentValue via DAC
    LATG1_bit = 0;                             // CS enable for DAC
    SPI2_Write(0x3000 | CurrentValue);         // Write CurrentValue to DAC (0x3 is required by DAC)
    LATG1_bit  = 1;                            // CS disable for DAC
  }
}