/*
 * Project name:
     CFlash_Common (Basic use of Compact Flash common library)
 * Copyright:
     (c) MikroElektronika, 2012.
 * Revision History:
     20091030:
       - initial release;
 * Description:
     This project is a simple demonstration of CF(Compact Flash) Library which
     shows how to use CF card ( http://en.wikipedia.org/wiki/Compact_flash )
     data accessing routines. With minor adjustments,
     this example should work with any other MCU.
     Data access routines need two ports for communication with CF card.
     Before write operation make sure that you do not overwrite boot or FAT
     sector, because it could make your card unreadable by a PC or by a digital
     photo camera.
 * Test configuration:
     MCU:             dsPIC33FJ256GP710A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70593d.pdf
     Dev.Board:       EasyPIC Fusion v7
                      http://www.mikroe.com/easypic-fusion/
     Oscillator:      XT-PLL8, 80.000MHz
     Ext. Modules:    mE ac:C_Flash Board on PORTD(Control) and PORTF(Data)
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Turn ON PORTB LEDs at SW15. (board specific)
     - This example does NOT cover writing/reading of FILES on the CF media. This
       topic is covered in CF_FAT16 examples.
     - For some CF cards external power supply may be required.
 */

// set compact flash pinout
char Cf_Data_Port at PORTF;

sbit CF_RDY at RD7_bit;
sbit CF_WE  at RD6_bit;
sbit CF_OE  at RD5_bit;
sbit CF_CD1 at RD4_bit;
sbit CF_CE1 at RD3_bit;
sbit CF_A2  at RD2_bit;
sbit CF_A1  at RD1_bit;
sbit CF_A0  at RD0_bit;

sbit CF_RDY_direction at TRISD7_bit;
sbit CF_WE_direction  at TRISD6_bit;
sbit CF_OE_direction  at TRISD5_bit;
sbit CF_CD1_direction at TRISD4_bit;
sbit CF_CE1_direction at TRISD3_bit;
sbit CF_A2_direction  at TRISD2_bit;
sbit CF_A1_direction  at TRISD1_bit;
sbit CF_A0_direction  at TRISD0_bit;
// end of cf pinout

char SignalPort           at PORTB;
char SignalPort_direction at TRISB;

void InitCF() {
  CF_CD1_direction = 1;
  while (Cf_Detect() == 0)                // wait until CF card is inserted
    ;
  Cf_Init();                              // initialize CF
  while (!CF_RDY)
    ;
  Delay_ms(2000);                         // wait for a while until the card is stabilized
}                                         //   period depends on used CF card

void TestBytes() {
  unsigned int i;

  ///// Write numbers 0..511 to sector 590
  Cf_Write_Init(590,1);                   // Initialize write at sector address 590
                                          //     for 1 sector
  SignalPort = 0x03;                      // Notify that write has started
  Delay_ms(1000);
  for (i=0; i<=511; i++)                  // Write 512 bytes to sector 590
    Cf_Write_Byte(i);

  SignalPort = 0x07;                      // Notify that write end and read start
  Delay_ms(1000);

  Cf_Read_Init(590,1);                    // Initialize read from sector address 590
                                          //    for 1 sector
  for (i=0; i<=511; i++) {                // Read 512 bytes from sector address 590
    SignalPort = Cf_Read_Byte();          // Read one byte at time and display
                                          //    readings on signal port
    Delay_ms(5);                          // Wait for a while to see results
  }
  Delay_ms(1000);

  ///// Write numbers 511..0 to sector 590
  Cf_Write_Init(590,1);                   // Initialize write at sector address 590
                                          //     for 1 sector
  SignalPort = 0x03;                      // Notify that write has started
  Delay_ms(1000);
  for (i=0; i<=511; i++)                  // Write 512 bytes to sector 590
    Cf_Write_Byte(511-i);

  SignalPort = 0x07;                      // Notify that write end and read start
  Delay_ms(1000);

  Cf_Read_Init(590,1);                    // Initialize read from sector address 590
                                          //    for 1 sector
  for (i=0; i<=511; i++) {                // Read 512 bytes from sector address 590
    SignalPort = Cf_Read_Byte();          // Read one byte at time and display
                                          //    readings on signal port
    Delay_ms(5);                          // Wait for a while to see results
  }
  Delay_ms(1000);
}

// Main program
void main() {

  ADPCFG = 0xFFFF;                        // disable A/D inputs

  SignalPort_direction = 0;               // designate PORTC as output
  SignalPort = 0x01;                      // Notify test start
  InitCF();

  TestBytes();
  SignalPort = 0x0F;                      // Notify test end
}