/*
 * Project name:
     Soft_SPI (Driving external Digital-to-Analog converter)
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Revision History:
     20091106:
       - initial release;
 * Description:
     This is a sample program which demonstrates the use of the Microchip's
     MCP4921 12-bit D/A converter with dsPIC MCUs. This device accepts digital
     input (number from 0..4095) and transforms it to the output voltage,
     ranging from 0..Vref. In this example the DAC communicates with MCU
     through the Soft_SPI communication.
     Buttons RB0 and RB1 are used to change value sent to DAC.
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EasyPIC v7 for dsPIC30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL8, 10.000MHz
     Ext. Modules:    mE ac:DAC extra board on PORTF
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Turn on dsPIC and CS0 switches on DAC extra board.
     - Put button jumper (J15) into VCC position and pull-down PORTB. (board specific)
 */

// DAC module connections
sbit Chip_Select at LATF0_bit;
sbit SoftSpi_CLK at LATF6_bit;
sbit SoftSpi_SDI at RF2_bit;
sbit SoftSpi_SDO at LATF3_bit;

sbit Chip_Select_Direction at TRISF0_bit;
sbit SoftSpi_CLK_Direction at TRISF6_bit;
sbit SoftSpi_SDI_Direction at TRISF2_bit;
sbit SoftSpi_SDO_Direction at TRISF3_bit;
// End DAC module connections

unsigned int value;

void InitMain() {
  TRISB0_bit = 1;                        // Set RB0 pin as input
  TRISB1_bit = 1;                        // Set RB1 pin as input
  Chip_Select = 1;                       // Deselect DAC
  Chip_Select_Direction = 0;             // Set CS# pin as Output
  Soft_SPI_Init();                       // Initialize Soft_SPI
}

// DAC increments (0..4095) --> output voltage (0..Vref)
void DAC_Output(unsigned int valueDAC) {
  char temp;

  Chip_Select = 0;                       // Select DAC chip

  // Send High Byte
  temp = (valueDAC >> 8) & 0x0F;         // Store valueDAC[11..8] to temp[3..0]
  temp |= 0x30;                          // Define DAC setting, see MCP4921 datasheet
  Soft_SPI_Write(temp);                  // Send high byte via Soft SPI

  // Send Low Byte
  temp = valueDAC;                       // Store valueDAC[7..0] to temp[7..0]
  Soft_SPI_Write(temp);                  // Send low byte via Soft SPI

  Chip_Select = 1;                       // Deselect DAC chip
}

void main() {

  ADPCFG = 0xFFFF;                       // Configure AN pins as digital

  InitMain();                            // Perform main initialization

  value = 2048;                          // When program starts, DAC gives
                                         //   the output in the mid-range

  while (1) {                            // Endless loop

    if ((RB0_bit) && (value < 4095)) {   // If RB0 button is pressed
      value++;                           //   increment value
      }
    else {
      if ((RB1_bit) && (value > 0)) {    // If RB1 button is pressed
        value--;                         //   decrement value
        }
      }

    DAC_Output(value);                   // Send value to DAC chip
    Delay_ms(1);                         // Slow down key repeat pace
  }
}