/*
 * Project Name:
     SPI_T6963C_240x128 (SPI_Glcd Library demo for Toshiba's T6963 Controller)
 * Copyright:
     (c) Mikroelektronika, 2012.
 * Revision History:
     20091106:
       - initial release; Credits to Bruno Gavand.
 * Description:
     This code in intended to work with GLCD's based on TOSHIBA T6963C controller
     through SPI interface. Pressing buttons RB8 .. RB12 and RC13 generates commands for text and graphic displaying.
     This parts may need a -15V power supply on Vee for LCD drive,
     a simple DC/DC converter can be made with a 2N2905, 220 H self, diode & 47 F cap,
     transistor base is driven with PWM through a current limiting resistor.
     This parts have a 8 Kb built-in display RAM, this allows 2 graphics panels
     and one text panel.
 * Test configuration:
     MCU:             dsPIC30F4013
                      http://ww1.microchip.com/downloads/en/DeviceDoc/70138F.pdf
     Dev.Board:       EasyPIC v7 for dsPIC30
                      http://www.mikroe.com/easypic-dspic30/
     Oscillator:      XT-PLL8, 10.000MHz
     Ext. Modules:    mE ac:SPI_T6963C 240x128 adapter on PORTF
                      T6963C display 240x128 pixels
                      http://www.mikroe.com/en/tools/components/#other
                      SmartADAPT Extra Development Board.
                      http://www.mikroe.com/add-on-boards/various/smartadapt/
     SW:              mikroC PRO for dsPIC30/33 and PIC24
                      http://www.mikroe.com/mikroc/dspic/
 * NOTES:
     - Put button jumper (J15) into Vcc position and pull-down PORTB and PORTC at SW2 (board specific).
 */
#include "__SPIT6963C.h"

/*
 * bitmap pictures stored in ROM
 */
const code char mikroE_240x128_bmp[];
const code char einstein[];

// Port Expander module connections
sbit  SPExpanderRST at LATF0_bit;     // for writing to output pin always use latch
sbit  SPExpanderCS  at LATF1_bit;     // for writing to output pin always use latch
sbit  SPExpanderRST_Direction at TRISF0_bit;
sbit  SPExpanderCS_Direction  at TRISF1_bit;
// End Port Expander module connections

void main() {

  #define COMPLETE_EXAMPLE
  #define LINE_DEMO           // Uncomment to demonstrate line drawing routines
  #define FILL_DEMO           // Uncomment to demonstrate fill routines
  #define PARTIAL_IMAGE_DEMO  // Uncomment to demonstrate partial image routine
  
  char txt1[] = " EINSTEIN WOULD HAVE LIKED mE";
  char txt[] =  " GLCD LIBRARY DEMO, WELCOME !";
  char txt2[] = "Partial image demo!";

  unsigned char  panel;         // Current panel
  unsigned int   i;             // General purpose register
  unsigned char  curs;          // Cursor visibility
  unsigned int   cposx, cposy;  // Cursor x-y position

  ADPCFG = 0xFFFF;

  TRISB8_bit = 1;               // Set RB8 as input
  TRISB9_bit = 1;               // Set RB9 as input
  TRISB10_bit = 1;              // Set RB10 as input
  TRISB11_bit = 1;              // Set RB11 as input
  TRISB12_bit = 1;              // Set RB12 as input
  TRISC13_bit = 1;              // Set RC13 as input

  // If Port Expander Library uses SPI1 module
  // Initialize SPI module used with PortExpander
  SPI1_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, _SPI_PRESCALE_SEC_1, _SPI_PRESCALE_PRI_4, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_IDLE_2_ACTIVE);

  // If Port Expander Library uses SPI2 module
  // SPI2_Init_Advanced(_SPI_MASTER, _SPI_8_BIT, _SPI_PRESCALE_SEC_1, _SPI_PRESCALE_PRI_4, _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_MIDDLE, _SPI_CLK_IDLE_LOW, _SPI_IDLE_2_ACTIVE);

  /*
   * init display for 240 pixel width and 128 pixel height
   * 8 bits character width
   * data bus on MCP23S17 portB
   * control bus on MCP23S17 portA
   * bit 2 is !WR
   * bit 1 is !RD
   * bit 0 is !CD
   * bit 4 is RST
   * chip enable, reverse on, 8x8 font internaly set in library
   */

  SPI_T6963C_Config(240, 128, 8, 0, 2, 1, 0, 4);
  Delay_ms(1000);

  /*
   * Enable both graphics and text display at the same time
   */
  SPI_T6963C_graphics(1);
  SPI_T6963C_text(1);

  panel = 0;
  i = 0;
  curs = 0;
  cposx = cposy = 0;
 
  /*
   * Text messages
   */
  SPI_T6963C_write_text(txt, 0, 0, SPI_T6963C_ROM_MODE_XOR);
  SPI_T6963C_write_text(txt1, 0, 15, SPI_T6963C_ROM_MODE_XOR);

  /*
   * Cursor
   */
  SPI_T6963C_cursor_height(8);       // 8 pixel height
  SPI_T6963C_set_cursor(0, 0);       // move cursor to top left
  SPI_T6963C_cursor(0);              // cursor off

  /*
   * Draw solid boxes
   */
  SPI_T6963C_box(0, 0, 239, 8, SPI_T6963C_WHITE);
  SPI_T6963C_box(0, 119, 239, 127, SPI_T6963C_WHITE);

  /*
   * Draw rectangles
   */
  #ifdef LINE_DEMO
    SPI_T6963C_rectangle(0, 0, 239, 127, SPI_T6963C_WHITE);
    SPI_T6963C_rectangle(20, 20, 219, 107, SPI_T6963C_WHITE);
    SPI_T6963C_rectangle(40, 40, 199, 87, SPI_T6963C_WHITE);
    SPI_T6963C_rectangle(60, 60, 179, 67, SPI_T6963C_WHITE);
  #endif

  /*
   * Draw rounded edge rectangle
   */
  #ifdef LINE_DEMO
    SPI_T6963C_Rectangle_Round_Edges(10, 10, 229, 117, 12, SPI_T6963C_WHITE);
    SPI_T6963C_Rectangle_Round_Edges(30, 30, 209, 97, 12, SPI_T6963C_WHITE);
    SPI_T6963C_Rectangle_Round_Edges(50, 50, 189, 77, 12, SPI_T6963C_WHITE);
  #endif

  /*
   * Draw filled rounded edge rectangle
   */
  #ifdef FILL_DEMO
    SPI_T6963C_Rectangle_Round_Edges_Fill(10, 10, 229, 117, 12, SPI_T6963C_WHITE);
    SPI_T6963C_Rectangle_Round_Edges_Fill(20, 20, 219, 107, 12, SPI_T6963C_BLACK);
    SPI_T6963C_Rectangle_Round_Edges_Fill(30, 30, 209, 97, 12, SPI_T6963C_WHITE);
    SPI_T6963C_Rectangle_Round_Edges_Fill(40, 40, 199, 87, 12, SPI_T6963C_BLACK);
    SPI_T6963C_Rectangle_Round_Edges_Fill(50, 50, 189, 77, 12, SPI_T6963C_WHITE);
  #endif

  /*
   * Draw a cross
   */
  #ifdef LINE_DEMO
    SPI_T6963C_line(0, 0, 239, 127, SPI_T6963C_WHITE);
    SPI_T6963C_line(0, 127, 239, 0, SPI_T6963C_WHITE);
  #endif
  
  /*
   * Draw circles
   */
  #ifdef LINE_DEMO
    SPI_T6963C_circle(120, 64, 10, SPI_T6963C_WHITE);
    SPI_T6963C_circle(120, 64, 30, SPI_T6963C_WHITE);
    SPI_T6963C_circle(120, 64, 50, SPI_T6963C_WHITE);
    SPI_T6963C_circle(120, 64, 70, SPI_T6963C_WHITE);
    SPI_T6963C_circle(120, 64, 90, SPI_T6963C_WHITE);
    SPI_T6963C_circle(120, 64, 110, SPI_T6963C_WHITE);
    SPI_T6963C_circle(120, 64, 130, SPI_T6963C_WHITE);
  #endif

  /*
   * Draw filled circles
   */
  #ifdef FILL_DEMO
    SPI_T6963C_circle_fill(120, 64, 60, SPI_T6963C_WHITE);
    SPI_T6963C_circle_fill(120, 64, 55, SPI_T6963C_BLACK);
    SPI_T6963C_circle_fill(120, 64, 50, SPI_T6963C_WHITE);
    SPI_T6963C_circle_fill(120, 64, 45, SPI_T6963C_BLACK);
    SPI_T6963C_circle_fill(120, 64, 40, SPI_T6963C_WHITE);
    SPI_T6963C_circle_fill(120, 64, 35, SPI_T6963C_BLACK);
    SPI_T6963C_circle_fill(120, 64, 30, SPI_T6963C_WHITE);
    SPI_T6963C_circle_fill(120, 64, 25, SPI_T6963C_BLACK);
    SPI_T6963C_circle_fill(120, 64, 20, SPI_T6963C_WHITE);
    SPI_T6963C_circle_fill(120, 64, 15, SPI_T6963C_BLACK);
    SPI_T6963C_circle_fill(120, 64, 10, SPI_T6963C_WHITE);
    SPI_T6963C_circle_fill(120, 64, 5, SPI_T6963C_BLACK);
  #endif
  
  Delay_ms(1000);
  SPI_T6963C_sprite(76, 4, einstein, 88, 119);  // Draw a sprite
  Delay_ms(1000);

  SPI_T6963C_setGrPanel(1);                     // Select other graphic panel

  SPI_T6963C_image(mikroE_240x128_bmp);
  SPI_T6963C_displayGrPanel(1);
  Delay_ms(1000);
  #ifdef PARTIAL_IMAGE_DEMO
    SPI_T6963C_grFill(0);
    SPI_T6963C_PartialImage(0, 0, 64, 64, 240, 128, mikroE_240x128_bmp);  // Display partial image
    Delay_ms(1000);
    SPI_T6963C_graphics(0);
  #endif
  SPI_T6963C_image(mikroE_240x128_bmp);
  SPI_T6963C_graphics(1);
  SPI_T6963C_displayGrPanel(0);

  for(;;) {                                               // Endless loop
     /*
     * If RB8 is pressed, display only graphic panel
     */
    if(RB8_bit) {
      SPI_T6963C_graphics(1);
      SPI_T6963C_text(0);
      Delay_ms(300);
      }
    #ifdef COMPLETE_EXAMPLE
      /*
       * If RB9 is pressed, toggle the display between graphic panel 0 and graphic panel 1
       */
      else if(RB9_bit) {
        panel++;
        panel &= 1;
        SPI_T6963C_displayGrPanel(panel);
        Delay_ms(300);
        }
    #endif
    /*
     * If RB10 is pressed, display only text panel
     */
    else if(RB10_bit) {
      SPI_T6963C_graphics(0);
      SPI_T6963C_text(1);
      Delay_ms(300);
      }

    /*
     * If RB11 is pressed, display text and graphic panels
     */
    else if(RB11_bit) {
      SPI_T6963C_graphics(1);
      SPI_T6963C_text(1);
      Delay_ms(300);
      }

    /*
     * If RB12 is pressed, change cursor
     */
    else if(RB12_bit) {
      curs++;
      if(curs == 3) curs = 0;
      switch(curs) {
        case 0:
          // no cursor
          SPI_T6963C_cursor(0);
          break;
        case 1:
          // blinking cursor
          SPI_T6963C_cursor(1);
          SPI_T6963C_cursor_blink(1);
          break;
        case 2:
          // non blinking cursor
          SPI_T6963C_cursor(1);
          SPI_T6963C_cursor_blink(0);
          break;
        }
      Delay_ms(300);
      }

    #ifdef PARTIAL_IMAGE_DEMO
     /*
     * If RC13 is pressed, perform the "Partial image" demostration
     */
    else if(RC13_bit) {
      SPI_T6963C_setGrPanel(0);
      SPI_T6963C_setTxtPanel(0);
      SPI_T6963C_txtFill(0);
      SPI_T6963C_setGrPanel(1);
      SPI_T6963C_setTxtPanel(0);
      SPI_T6963C_graphics(1);
      SPI_T6963C_text(1);
      SPI_T6963C_displayGrPanel(1);
      SPI_T6963C_write_text(txt2, 5, 15, SPI_T6963C_ROM_MODE_XOR);
      Delay_1sec();
      SPI_T6963C_grFill(0);
      SPI_T6963C_PartialImage(0, 0, 64, 64, 240, 128, mikroE_240x128_bmp);
      Delay_1sec();
      SPI_T6963C_PartialImage(0, 0, 128, 128, 240, 128, mikroE_240x128_bmp);
      Delay_1sec();
      SPI_T6963C_PartialImage(0, 0, 240, 128, 240, 128, mikroE_240x128_bmp);
      Delay_1sec();
      SPI_T6963C_txtFill(0);
      SPI_T6963C_write_text(txt, 0, 0, SPI_T6963C_ROM_MODE_XOR);
      SPI_T6963C_write_text(txt1, 0, 15, SPI_T6963C_ROM_MODE_XOR);
    }
    #endif

    /*
     * Move cursor, even if not visible
     */
    cposx++;
    if(cposx == SPI_T6963C_txtCols) {
      cposx = 0;
      cposy++;
      if(cposy == SPI_T6963C_grHeight / SPI_T6963C_CHARACTER_HEIGHT) {
        cposy = 0;
        }
      }
    SPI_T6963C_set_cursor(cposx, cposy);

    Delay_ms(100);

    }
}